import React, { ReactNode } from 'react';

type CommandContext = {
    setStdin: (s: string) => void;
    /**
     * Adds a new command to stdin and pushes it to the history
     */
    addCommand: (command: CommandResult) => void;
};
type Command = (ctx: CommandContext) => void | Promise<void>;
type CliProps = {
    url: string;
    token: string;
    welcome?: ReactNode;
    /**
     * trigger one or multiple commands to run on start
     *
     * For example `init: "help"` shows the help
     */
    init?: string | string[];
    /**
     * The className prop is used to add custom styles to the cli and applied to the root element
     */
    className?: string;
    commands?: Record<string, Command>;
};
type CommandResult = {
    /**
     * @default Date.now()
     */
    time?: number;
    command: string;
    result?: string | null | number | boolean | string[] | React.ReactNode;
    error?: boolean;
};
declare const RedisCli: React.FC<CliProps>;

export { CliProps, RedisCli };
