"use strict";Object.defineProperty(exports, "__esModule", {value: true}); function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { newObj[key] = obj[key]; } } } newObj.default = obj; return newObj; } } function _nullishCoalesce(lhs, rhsFn) { if (lhs != null) { return lhs; } else { return rhsFn(); } } function _optionalChain(ops) { let lastAccessLHS = undefined; let value = ops[0]; let i = 1; while (i < ops.length) { const op = ops[i]; const fn = ops[i + 1]; i += 2; if ((op === 'optionalAccess' || op === 'optionalCall') && value == null) { return undefined; } if (op === 'access' || op === 'optionalAccess') { lastAccessLHS = value; value = fn(value); } else if (op === 'call' || op === 'optionalCall') { value = fn((...args) => value.call(lastAccessLHS, ...args)); lastAccessLHS = undefined; } } return value; }// src/redis-cli.tsx
var _react = require('react');
var _reactscrollarea = require('@radix-ui/react-scroll-area'); var ScrollArea = _interopRequireWildcard(_reactscrollarea);
var _jsxruntime = require('react/jsx-runtime');
var RedisCli = (props) => {
  const [history, setHistory] = _react.useState.call(void 0, []);
  const [historyIndex, setHistoryIndex] = _react.useState.call(void 0, null);
  const [results, setResults] = _react.useState.call(void 0, []);
  function addCommand(command) {
    command.time ??= Date.now();
    setHistory((prev) => {
      if (prev.length === 0 || prev[0] !== command.command) {
        return [command.command, ...prev];
      }
      return prev;
    });
    setResults((prev) => [...prev, command]);
  }
  const [loading, setLoading] = _react.useState.call(void 0, false);
  const [stdin, setStdin] = _react.useState.call(void 0, "");
  const specialCommands = {
    clear: (_ctx) => {
      setResults([]);
      setHistoryIndex(null);
      setStdin("");
    },
    help: (ctx) => {
      ctx.addCommand({
        time: Date.now(),
        command: "help",
        result: /* @__PURE__ */ _jsxruntime.jsxs.call(void 0, "div", { children: [
          /* @__PURE__ */ _jsxruntime.jsx.call(void 0, "p", { children: "You can execute Redis commands in the terminal:" }),
          /* @__PURE__ */ _jsxruntime.jsx.call(void 0, 
            "a",
            {
              className: "upstash-cli-link",
              href: "https://upstash.com/redis-api-compatibility",
              target: "_blank",
              rel: "noreferrer",
              children: "https://upstash.com/redis-api-compatibility"
            }
          ),
          /* @__PURE__ */ _jsxruntime.jsx.call(void 0, "br", {}),
          /* @__PURE__ */ _jsxruntime.jsxs.call(void 0, "div", { className: "upstash-cli-help-command", children: [
            /* @__PURE__ */ _jsxruntime.jsx.call(void 0, "span", { children: "Try" }),
            /* @__PURE__ */ _jsxruntime.jsx.call(void 0, "button", { className: "upstash-cli-code", onClick: () => ctx.setStdin("GET key"), children: "> GET key" }),
            /* @__PURE__ */ _jsxruntime.jsx.call(void 0, "button", { className: "upstash-cli-code", onClick: () => ctx.setStdin("LPUSH list e1 e2"), children: "> LPUSH list" })
          ] }),
          /* @__PURE__ */ _jsxruntime.jsxs.call(void 0, "div", { className: "upstash-cli-help-command", children: [
            /* @__PURE__ */ _jsxruntime.jsx.call(void 0, "span", { children: "Special commands" }),
            /* @__PURE__ */ _jsxruntime.jsxs.call(void 0, "button", { className: "upstash-cli-code", onClick: () => ctx.setStdin("clear"), children: [
              "> clear",
              " "
            ] }),
            /* @__PURE__ */ _jsxruntime.jsxs.call(void 0, "button", { className: "upstash-cli-code", onClick: () => ctx.setStdin("help"), children: [
              "> help",
              " "
            ] })
          ] })
        ] })
      });
    },
    ...props.commands
  };
  async function runRedisCommand(command) {
    try {
      const args = splitArgs(command);
      const res = await fetch(props.url, {
        method: "POST",
        headers: {
          "Content-Type": "application/json",
          Authorization: `Bearer ${props.token}`
        },
        body: JSON.stringify(args)
      });
      const json = await res.json();
      addCommand({
        command,
        result: _nullishCoalesce(json.error, () => ( json.result)),
        error: Boolean(json.error),
        time: Date.now()
      });
    } catch (error) {
      console.error(error);
      addCommand({ command, error: true, result: error.message, time: Date.now() });
    }
  }
  _react.useEffect.call(void 0, () => {
    if (historyIndex === null) {
      return;
    }
    const cmd = history[historyIndex];
    if (!cmd) {
      return;
    }
    setStdin(cmd);
  }, [historyIndex, history]);
  const ref = _react.useRef.call(void 0, null);
  const onEnter = async () => {
    try {
      if (loading) {
        return;
      }
      setLoading(true);
      const command = stdin.trim();
      if (!command) {
        addCommand({ command, time: Date.now() });
        return;
      }
      if (specialCommands[command]) {
        specialCommands[command]({
          setStdin,
          addCommand
        });
        return;
      }
      await runRedisCommand(command);
    } catch (error) {
      const err = error;
      console.error(err.message);
    } finally {
      setHistoryIndex(null);
      setLoading(false);
      setStdin("");
      setTimeout(() => _optionalChain([ref, 'access', _ => _.current, 'optionalAccess', _2 => _2.scrollIntoView, 'call', _3 => _3({ behavior: "smooth", block: "nearest" })]), 10);
    }
  };
  _react.useEffect.call(void 0, () => {
    async function run() {
      if (!props.init) {
        return;
      }
      try {
        const commands = Array.isArray(props.init) ? props.init : [props.init];
        for (const command of commands) {
          const cmd = specialCommands[command];
          if (cmd) {
            cmd({
              setStdin,
              addCommand
            });
          } else {
            await runRedisCommand(command);
          }
        }
      } catch (error) {
        console.error(error);
      }
    }
    run();
  }, [props.init]);
  return /* @__PURE__ */ _jsxruntime.jsx.call(void 0, 
    "div",
    {
      className: `upstash-cli ${props.className}`,
      onMouseUp: () => {
        if (_optionalChain([window, 'optionalAccess', _4 => _4.getSelection, 'call', _5 => _5(), 'optionalAccess', _6 => _6.type]) !== "Range") {
          _optionalChain([ref, 'access', _7 => _7.current, 'optionalAccess', _8 => _8.focus, 'call', _9 => _9()]);
        }
      },
      children: /* @__PURE__ */ _jsxruntime.jsxs.call(void 0, ScrollArea.Root, { style: { overflow: "hidden" }, children: [
        /* @__PURE__ */ _jsxruntime.jsxs.call(void 0, ScrollArea.Viewport, { className: "upstash-cli-viewport", children: [
          /* @__PURE__ */ _jsxruntime.jsx.call(void 0, "span", { style: { color: "#00e9a3" }, children: _nullishCoalesce(props.welcome, () => ( /* @__PURE__ */ _jsxruntime.jsxs.call(void 0, _jsxruntime.Fragment, { children: [
            "Welcome to Upstash CLI!",
            /* @__PURE__ */ _jsxruntime.jsx.call(void 0, "br", {}),
            "Execute Redis commands online (e.g. 'SET key value'). Type 'help' for more."
          ] }))) }),
          results.map((r) => /* @__PURE__ */ _jsxruntime.jsx.call(void 0, Result, { result: r }, r.time)),
          /* @__PURE__ */ _jsxruntime.jsx.call(void 0, Line, { className: loading ? "upstash-cli-loading" : "", prefix: /* @__PURE__ */ _jsxruntime.jsx.call(void 0, "span", { children: "\u279C" }), children: /* @__PURE__ */ _jsxruntime.jsx.call(void 0, 
            "input",
            {
              type: "text",
              spellCheck: false,
              style: {
                resize: "none"
              },
              ref,
              value: stdin,
              onChange: (e) => setStdin(e.currentTarget.value),
              onKeyDown: async (e) => {
                if (e.ctrlKey && e.key === "c") {
                  e.preventDefault();
                  addCommand({ command: `${stdin}^C`, time: Date.now() });
                  setStdin("");
                  return;
                }
                if (e.key === "Enter" && !e.shiftKey) {
                  e.preventDefault();
                  await onEnter();
                  return;
                }
                if (e.key === "ArrowUp") {
                  if (history.length === 0) {
                    setHistoryIndex(null);
                    return;
                  }
                  setHistoryIndex(
                    Math.min(history.length - 1, historyIndex === null ? 0 : historyIndex + 1)
                  );
                  return;
                }
                if (e.key === "ArrowDown") {
                  if (history.length === 0) {
                    setHistoryIndex(null);
                    return;
                  }
                  setHistoryIndex(
                    Math.max(0, historyIndex === null ? history.length - 1 : historyIndex - 1)
                  );
                  return;
                }
              },
              className: "upstash-cli-stdin"
            },
            "stdin"
          ) })
        ] }),
        /* @__PURE__ */ _jsxruntime.jsx.call(void 0, ScrollArea.Scrollbar, { className: "upstash-cli-scrollbar", orientation: "vertical", children: /* @__PURE__ */ _jsxruntime.jsx.call(void 0, ScrollArea.Thumb, { className: "upstash-cli-scrollbar-thumb" }) }),
        /* @__PURE__ */ _jsxruntime.jsx.call(void 0, ScrollArea.Corner, { style: { background: "black" } })
      ] })
    }
  );
};
var Line = ({
  className,
  prefix,
  children
}) => /* @__PURE__ */ _jsxruntime.jsxs.call(void 0, "div", { className: ["upstash-cli-line", className].join(" "), children: [
  /* @__PURE__ */ _jsxruntime.jsx.call(void 0, "div", { className: "upstash-cli-line-prefix", children: _nullishCoalesce(prefix, () => ( /* @__PURE__ */ _jsxruntime.jsx.call(void 0, "span", { children: " " }))) }),
  /* @__PURE__ */ _jsxruntime.jsx.call(void 0, "div", { className: "upstash-cli-line-content", children })
] });
function splitArgs(input) {
  const separator = /\s/g;
  let singleQuoteOpen = false;
  let doubleQuoteOpen = false;
  let tokenBuffer = [];
  const ret = [];
  const arr = [...input];
  for (const element of arr) {
    const matches = element.match(separator);
    if (element === "'" && !doubleQuoteOpen) {
      singleQuoteOpen = !singleQuoteOpen;
      continue;
    }
    if (element === '"' && !singleQuoteOpen) {
      doubleQuoteOpen = !doubleQuoteOpen;
      continue;
    }
    if (!(singleQuoteOpen || doubleQuoteOpen) && matches) {
      if (tokenBuffer.length > 0) {
        ret.push(tokenBuffer.join(""));
        tokenBuffer = [];
      } else {
        ret.push(element);
      }
    } else {
      tokenBuffer.push(element);
    }
  }
  if (tokenBuffer.length > 0) {
    ret.push(tokenBuffer.join(""));
  } else {
    ret.push("");
  }
  return ret;
}
var Result = ({ result }) => {
  return /* @__PURE__ */ _jsxruntime.jsxs.call(void 0, _jsxruntime.Fragment, { children: [
    /* @__PURE__ */ _jsxruntime.jsx.call(void 0, 
      Line,
      {
        prefix: result.error ? /* @__PURE__ */ _jsxruntime.jsx.call(void 0, 
          "span",
          {
            style: {
              color: "#EF4444"
            },
            children: "\u2717"
          }
        ) : /* @__PURE__ */ _jsxruntime.jsx.call(void 0, "span", { children: "\u279C" }),
        children: /* @__PURE__ */ _jsxruntime.jsx.call(void 0, "span", { children: result.command })
      }
    ),
    result.result === void 0 ? null : /* @__PURE__ */ _jsxruntime.jsx.call(void 0, Line, { children: /* @__PURE__ */ _jsxruntime.jsx.call(void 0, 
      "div",
      {
        className: "upstash-cli-result",
        style: {
          color: result.error ? "#EF4444" : void 0
        },
        children: formatResult(result.result)
      }
    ) })
  ] });
};
function formatResult(result) {
  switch (typeof result) {
    case "undefined": {
      return "";
    }
    case "boolean":
    case "number": {
      return result.toString();
    }
    case "object": {
      if (result === null) {
        return "nil";
      }
      if (Array.isArray(result)) {
        return result.map((e) => formatResult(e)).join(", ");
      }
      return result;
    }
    default: {
      return result;
    }
  }
}


exports.RedisCli = RedisCli;
